@@ -2,13 +2,14 @@ import unittest
 import os
 import warnings
 import pickle
+import pytest
 
 import skrf as rf
 import numpy as npy
 from numpy.random  import rand, uniform
 import pytest
 
-from skrf.calibration import OnePort, PHN, SDDL, TRL, SOLT, UnknownThru, EightTerm, TwoPortOnePath, EnhancedResponse,TwelveTerm, SixteenTerm, LMR16, terminate, determine_line, determine_reflect, NISTMultilineTRL
+from skrf.calibration import OnePort, PHN, SDDL, TRL, SOLT, UnknownThru, EightTerm, TwoPortOnePath, EnhancedResponse,TwelveTerm, SixteenTerm, LMR16, terminate, terminate_nport, determine_line, determine_reflect, NISTMultilineTRL, MultiportCal, MultiportSOLT
 
 from skrf import two_port_reflect
 from skrf.networkSet import NetworkSet
@@ -21,8 +22,8 @@ from skrf.media import DistributedCircuit
 global NPTS  
 NPTS = 1
 
-WG_lossless =  rf.RectangularWaveguide(rf.F(75,100,NPTS), a=100*rf.mil, z0=50)
-WG =  rf.RectangularWaveguide(rf.F(75,100,NPTS), a=100*rf.mil, z0=50, rho='gold')
+WG_lossless = rf.RectangularWaveguide(rf.F(75, 100, NPTS, unit='GHz'), a=100*rf.mil, z0=50)
+WG = rf.RectangularWaveguide(rf.F(75, 100, NPTS, unit='GHz'), a=100*rf.mil, z0=50, rho='gold')
 
 
 class DetermineTest(unittest.TestCase):
@@ -70,7 +71,7 @@ class DetermineTest(unittest.TestCase):
         [ self.assertEqual(k,l) for k,l in zip(self.r, r_found)]
 
 
-class CalibrationTest(object):
+class CalibrationTest:
     """
     This is the generic Calibration test case which all Calibration 
     Subclasses should be able to pass. They must implement
@@ -145,6 +146,24 @@ class OnePortTest(unittest.TestCase, CalibrationTest):
             measured = measured,
             )
         
+    def test_input_networks_1port(self):
+        # test users do not enter 2-port networks by accident
+        with self.assertRaises(RuntimeError):
+            wg = self.wg
+            ideals = [
+                    two_port_reflect(wg.short( name='short'), wg.short( name='short')),
+                    wg.delay_short( 45.,'deg',name='ew'),
+                    wg.delay_short( 90.,'deg',name='qw'),
+                    wg.match( name='load'),
+                    ]
+            measured = [self.measure(k) for k in ideals]
+            cal = rf.OnePort(
+               is_reciprocal = True, 
+               ideals = ideals, 
+               measured = measured,
+               )
+            cal.run()
+
     def measure(self, ntwk):
         out = self.E**ntwk
         out.name = ntwk.name
@@ -202,7 +221,7 @@ class SDDLTest(OnePortTest):
     
     def test_init_with_nones(self):
         wg=self.wg
-        wg.frequency = rf.F.from_f([100])
+        wg.frequency = rf.F.from_f([100], unit='GHz')
         
         self.E = wg.random(n_ports =2, name = 'E')
         
@@ -1231,7 +1250,8 @@ class LRRMTest(EightTermTest):
         o_i = wg.load(1, nports=1, name='open')
         s_i = wg.short(nports=1, name='short')
         m_i = wg.load(0.1, nports=1, name='load')
-        thru = wg.line(d=50, z0=75, unit='um', name='thru', embed=True)
+        with pytest.warns(FutureWarning, match="`embed` will be deprecated"):
+            thru = wg.line(d=50, z0=75, unit='um', name='thru', embed=True)
         # Make sure calibration works with non-symmetric thru
         thru.s[:,1,1] += 0.02 + 0.05j
 
@@ -1414,7 +1434,7 @@ class TwelveTermToEightTermTest(unittest.TestCase, CalibrationTest):
     def setUp(self):
         self.n_ports = 2
         wg= rf.wr10
-        wg.frequency = rf.F.from_f([100])
+        wg.frequency = rf.F.from_f([100], unit='GHz')
         self.wg = wg
         self.X = wg.random(n_ports =2, name = 'X')
         self.Y = wg.random(n_ports =2, name='Y')
@@ -1795,6 +1815,170 @@ class LMR16Test(SixteenTermTest):
             self.thru,
             self.cal.solved_through)
 
-    
+class MultiportCalTest(unittest.TestCase):
+    """Multi-port NISTMultilineTRL calibration test"""
+
+    def test_cal(self):
+        self.wg = WG
+        wg = self.wg
+
+        self.n_ports = 3
+        nports = self.n_ports
+
+        self.make_error_networks(wg, nports)
+
+        cal_dict = {}
+
+        actuals = [
+            wg.thru(),
+            rf.two_port_reflect(wg.load(-.98-.1j),wg.load(-.98-.1j)),
+            wg.line(100,'um'),
+            wg.line(900,'um'),
+            ]
+
+        dut = self.wg.random(n_ports=self.n_ports, name='dut')
+        dut_meas = self.measure(dut)
+
+        l = [0, 100e-6, 900e-6]
+
+        for p in [(0, 1), (0, 2)]:
+
+            measured = [self.measure(rf.twoport_to_nport(k, p[0], p[1], nports)) for k in actuals]
+            cal_dict[p] = {}
+            cal_dict[p]['method'] = NISTMultilineTRL
+            cal_dict[p]['measured'] = measured
+            # Two-port switch terms are in reverse order from multi-port switch terms.
+            cal_dict[p]['switch_terms'] = [self.gammas[i] for i in p][::-1]
+            cal_dict[p]['l'] = l
+            cal_dict[p]['Grefls'] = [-1]
+            cal_dict[p]['er_est'] = 1
+            cal_dict[p]['gamma_root_choice'] = 'real'
+
+        isolation = self.measure(wg.match(nports=nports))
+
+        self.cal = MultiportCal(cal_dict, isolation=isolation)
+
+        # Test coefs
+        nports = self.n_ports
+        for e, c in enumerate(self.cal.coefs):
+            self.assertTrue(npy.abs(c['directivity'] - self.Z.s[:,e,e]) < 1e-7)
+            self.assertTrue(npy.abs(c['source match'] - self.Z.s[:,nports+e,nports+e]) < 1e-7)
+            self.assertTrue(npy.abs(c['reflection tracking'] - self.Z.s[:,e,nports+e] * self.Z.s[:,nports+e,e]) < 1e-7)
+            self.assertTrue(npy.abs(c['switch term'] - self.gammas[e].s) < 1e-7)
+            self.assertTrue(npy.abs(c['k']/self.cal.coefs[0]['k'] - self.Z.s[:,nports+0,0]/self.Z.s[:,nports+e,e]) < 1e-7)
+
+        # Test DUT correction
+        dut_cal = self.cal.apply_cal(dut_meas)
+        self.assertEqual(dut_cal, dut)
+
+        # Test cal inverts embed
+        a = self.wg.random(n_ports=self.n_ports)
+        self.assertEqual(self.cal.apply_cal(self.cal.embed(a)),a)
+
+        # Test embed equals measure
+        a = self.wg.random(n_ports=self.n_ports)
+        self.assertEqual(self.cal.embed(a),self.measure(a))
+
+        # Test gamma solved by TRL
+        for p in [(0, 1), (0, 2)]:
+            self.assertTrue(max(npy.abs(self.wg.gamma-self.cal.cals[p].gamma)) < 1e-3)
+
+    def make_error_networks(self, wg, nports):
+        self.Z = wg.random(n_ports = 2*nports, name = 'Z')
+
+        # Isolation terms are between all ports.
+        # No error in the same port.
+        self.isolation = wg.random(n_ports=nports, name='I')
+        for i in range(nports):
+            self.isolation.s[:, i, i] = 0
+
+        port_type = lambda n: 'VNA' if n < nports else 'DUT'
+        port_number = lambda n: n if n < nports else n - nports
+        # Remove leakage terms
+        for i in range(2*nports):
+            for j in range(i+1, 2*nports):
+                # No connection between different VNA/DUT ports.
+                # No connection between VNA and DUT ports with different number.
+                if port_type(i) == port_type(j) or port_number(i) != port_number(j):
+                    self.Z.s[:,i,j] = 0
+                    self.Z.s[:,j,i] = 0
+
+        self.gammas = []
+        for i in range(nports):
+            self.gammas.append(wg.random(n_ports=1, name=f'gamma_{i}'))
+
+    def terminate(self, ntwk):
+        """
+        Terminate a measured network with the switch terms
+        """
+        return terminate_nport(ntwk, self.gammas)
+
+    def measure(self, ntwk):
+        out = self.terminate(rf.connect(self.Z, self.n_ports, ntwk, 0, num=self.n_ports))
+        out = out + self.isolation
+        out.name = ntwk.name
+        return out
+
+class MultiportSOLTTest(MultiportCalTest):
+
+    def test_cal(self):
+        nport_list = [3, 4]
+        method_list = [SOLT, UnknownThru]
+
+        self.wg = WG
+        wg = self.wg
+
+        for nport in nport_list:
+            self.n_ports = nport
+            nports = self.n_ports
+
+            self.make_error_networks(wg, nports)
+
+            o = wg.open(nports=nports, name='open')
+            s = wg.short(nports=nports, name='short')
+            m = wg.match(nports=nports, name='load')
+
+            thru = wg.thru(name='thru')
+
+            ideals = []
+            # nports-1 thrus from port 0 to all other ports.
+            for i in range(1, nports):
+                thru_i = rf.twoport_to_nport(thru, 0, i, nports)
+                ideals.append(thru_i)
+
+            ideals.extend([o,s,m])
+            measured = [self.measure(k) for k in ideals]
+
+            dut = self.wg.random(n_ports=self.n_ports, name='dut')
+            dut_meas = self.measure(dut)
+
+            for method in method_list:
+
+                if method == SOLT:
+                    self.cal = MultiportSOLT(method, measured, ideals, isolation=measured[-1])
+                else:
+                    self.cal = MultiportSOLT(method, measured, ideals, isolation=measured[-1], switch_terms=self.gammas)
+
+                # Test coefs
+                nports = self.n_ports
+                for e, c in enumerate(self.cal.coefs):
+                    self.assertTrue(npy.abs(c['directivity'] - self.Z.s[:,e,e]) < 1e-7)
+                    self.assertTrue(npy.abs(c['source match'] - self.Z.s[:,nports+e,nports+e]) < 1e-7)
+                    self.assertTrue(npy.abs(c['reflection tracking'] - self.Z.s[:,e,nports+e] * self.Z.s[:,nports+e,e]) < 1e-7)
+                    self.assertTrue(npy.abs(c['switch term'] - self.gammas[e].s) < 1e-7)
+                    self.assertTrue(npy.abs(c['k']/self.cal.coefs[0]['k'] - self.Z.s[:,nports+0,0]/self.Z.s[:,nports+e,e]) < 1e-7)
+
+                # Test DUT correction
+                dut_cal = self.cal.apply_cal(dut_meas)
+                self.assertEqual(dut_cal, dut)
+
+                # Test cal inverts embed
+                a = self.wg.random(n_ports=self.n_ports)
+                self.assertEqual(self.cal.apply_cal(self.cal.embed(a)),a)
+
+                # Test embed equals measure
+                a = self.wg.random(n_ports=self.n_ports)
+                self.assertEqual(self.cal.embed(a),self.measure(a))
+
 if __name__ == "__main__":
     unittest.main()
