@@ -7,7 +7,7 @@ from skrf.calibration import OnePort, TRL, SOLT, EightTerm
 from skrf.util import suppress_warning_decorator
 
 
-class CalsetTest(object):
+class CalsetTest:
     @suppress_warning_decorator("No switch terms")
     def test_run(self):
         """
@@ -29,7 +29,7 @@ class DotOneport(unittest.TestCase,CalsetTest):
 
     """
     def setUp(self):
-        self.wg = rf.RectangularWaveguide(rf.F(75,100,11), a=100*rf.mil,z0=50)
+        self.wg = rf.RectangularWaveguide(rf.F(75, 100, 11, unit='GHz'), a=100*rf.mil,z0=50)
         wg = self.wg
         self.n_ports = 1
         self.E = wg.random(n_ports =2, name = 'E')
@@ -62,9 +62,9 @@ class DotEightTerm(unittest.TestCase, CalsetTest):
     @suppress_warning_decorator("divide by zero encountered in true_divide")
     def setUp(self):
         self.n_ports = 2
-        self.wg = rf.RectangularWaveguide(rf.F(75,100,3), a=100*rf.mil,z0=50)
+        self.wg = rf.RectangularWaveguide(rf.F(75, 100, 3, unit='GHz'), a=100*rf.mil,z0=50)
         wg= self.wg
-        wg.frequency = rf.F.from_f([100])
+        wg.frequency = rf.F.from_f([100], unit='GHz')
         
         self.X = wg.random(n_ports =2, name = 'X')
         self.Y = wg.random(n_ports =2, name='Y')
