@@ -1,3 +1,4 @@
+import pytest
 import skrf as rf
 import unittest
 import os
@@ -9,7 +10,7 @@ class DeembeddingTestCase(unittest.TestCase):
     """
     Testcase for the Deembedding class
     Pseudo-netlists for s-parameter files used in these tests
-    For open-short, open and short de-embedding:
+    For open-short de-embedding:
     - deemb_ind.s2p
         P1      (1 0) port
         Cpad1   (1 0) capacitor c=25fF
@@ -35,6 +36,20 @@ class DeembeddingTestCase(unittest.TestCase):
         Cpad2   (4 0) capacitor c=25fF
         Cp2p    (1 4) capacitor c=10fF
         P2      (4 0) port
+    For open de-embedding:
+    - deemb_ind7.s2p
+        P1      (1 0) port
+        Cpad1   (1 0) capacitor c=25fF
+        Dut_ind (1 2) inductor l=1nH
+        Cpad2   (2 0) capacitor c=25fF
+        Cp2p    (1 2) capacitor c=10fF
+        P2      (2 0) port
+    - deemb_open7.s2p
+        P1      (1 0) port
+        Cpad1   (1 0) capacitor c=25fF
+        Cpad2   (2 0) capacitor c=25fF
+        Cp2p    (1 2) capacitor c=10fF
+        P2      (2 0) port
     For short-open de-embedding:
     - deemb_ind2.s2p
         P1      (1 0) port
@@ -61,6 +76,18 @@ class DeembeddingTestCase(unittest.TestCase):
         Cp2p    (0 0) capacitor c=10fF
         Rline2  (0 4) resistor r=2ohm
         P2      (4 0) port
+    For short de-embedding:
+    - deemb_ind8.s2p
+        P1      (1 0) port
+        Rline1  (1 2) resistor r=2ohm
+        Dut_ind (2 3) inductor l=1nH
+        Rline2  (3 4) resistor r=2ohm
+        P2      (4 0) port
+    - deemb_short8.s2p
+        P1      (1 0) port
+        Rline1  (1 0) resistor r=2ohm
+        Rline2  (0 4) resistor r=2ohm
+        P2      (4 0) port
     For SplitPi de-embedding:
     - deemb_ind3.s2p
         P1      (1 0) port
@@ -125,7 +152,7 @@ class DeembeddingTestCase(unittest.TestCase):
         """
         self.test_dir = os.path.dirname(os.path.abspath(__file__))+'/deembed/'
         
-        # for open-short, open and short testing
+        # for open-short testing
         self.raw = rf.Network(os.path.join(self.test_dir, 'deemb_ind.s2p'))
         self.open = rf.Network(os.path.join(self.test_dir, 'deemb_open.s2p'))
         self.short = rf.Network(os.path.join(self.test_dir, 'deemb_short.s2p'))
@@ -135,6 +162,14 @@ class DeembeddingTestCase(unittest.TestCase):
         self.open_1f = self.open['10GHz'] 
         self.short_1f = self.short['10GHz']
 
+        # for open testing
+        self.raw7 = rf.Network(os.path.join(self.test_dir, 'deemb_ind7.s2p'))
+        self.open7 = rf.Network(os.path.join(self.test_dir, 'deemb_open7.s2p'))
+
+        # for spot frequency checking
+        self.raw7_1f = self.raw7['10GHz'] 
+        self.open7_1f = self.open7['10GHz'] 
+
         # for short-open testing
         self.raw2 = rf.Network(os.path.join(self.test_dir, 'deemb_ind2.s2p'))
         self.open2 = rf.Network(os.path.join(self.test_dir, 'deemb_open2.s2p'))
@@ -145,6 +180,14 @@ class DeembeddingTestCase(unittest.TestCase):
         self.open2_1f = self.open2['10GHz'] 
         self.short2_1f = self.short2['10GHz']
 
+        # for short testing
+        self.raw8 = rf.Network(os.path.join(self.test_dir, 'deemb_ind8.s2p'))
+        self.short8 = rf.Network(os.path.join(self.test_dir, 'deemb_short8.s2p'))
+        
+        # for spot frequency checking
+        self.raw8_1f = self.raw8['10GHz'] 
+        self.short8_1f = self.short8['10GHz']
+
         # for spilit-pi testing
         self.raw3 = rf.Network(os.path.join(self.test_dir, 'deemb_ind3.s2p'))
         self.thru3 = rf.Network(os.path.join(self.test_dir, 'deemb_thru3.s2p'))
@@ -180,27 +223,27 @@ class DeembeddingTestCase(unittest.TestCase):
         # create de-embedding objects
         self.dm = rf.OpenShort(self.open, self.short)
         self.dm_os = rf.OpenShort(self.open_1f, self.short_1f) 
-        self.dm_o = rf.Open(self.open_1f)
-        self.dm_s = rf.Short(self.short_1f)
+        self.dm_o = rf.Open(self.open7_1f)
         self.dm_so = rf.ShortOpen(self.short2_1f, self.open2_1f)
+        self.dm_s = rf.Short(self.short8_1f)
         self.dm_pi = rf.SplitPi(self.thru3_1f)
         self.dm_tee = rf.SplitTee(self.thru4_1f)
         self.dm_ac = rf.AdmittanceCancel(self.thru5_1f)
         self.dm_ic = rf.ImpedanceCancel(self.thru6_1f)
         
         # relative tolerance for comparisons
-        self.rtol = 1e-3
+        self.rtol = 1e-6
 
     def test_freqmismatch(self):
         """
-        Check that error is caught when networks are of different frequencies
+        Check that a warning is caught when networks are of different frequencies
         """
-        with self.assertRaises(ValueError):
-            rf.OpenShort(self.open, self.short_1f)
+        with self.assertWarns(RuntimeWarning):
+            rf.OpenShort(self.open, self.short[0:len(self.short)//2])
         
-        with self.assertRaises(ValueError):
-            self.dm_os.deembed(self.raw)
-
+        with self.assertWarns(RuntimeWarning):
+            self.dm.deembed(self.raw[0:len(self.raw)//2])
+            
     def test_openshort(self):
         """
         After de-embedding, the network is a pure inductor of 1nH.
@@ -212,35 +255,31 @@ class DeembeddingTestCase(unittest.TestCase):
 
     def test_open(self):
         """
-        After open de-embedding, the network is a R-L-R network with 2ohm-1nH-2ohm.
+        After de-embedding, the network is a pure inductor of 1nH.
         Test that this is true at a spot frequency.
         """
-        dut = self.dm_o.deembed(self.raw_1f)
-        res_calc = np.real(1/dut.y[0,0,0])
-        ind_calc = 1e9*np.imag(1/dut.y[0,0,0])/2/np.pi/dut.f
-        self.assertTrue(np.isclose(res_calc, 4, rtol=self.rtol))
-        self.assertTrue(np.isclose(ind_calc, 1, rtol=self.rtol))
-
-    def test_short(self):
-        """
-        First do open de-embedding, and next short. It should give pure inductor of 1nH.
-        This is similar to OpenShort, but done in 2 steps.
-        """
-        raw_minus_open = self.dm_o.deembed(self.raw_1f)
-        dut = self.dm_s.deembed(raw_minus_open)
+        dut = self.dm_o.deembed(self.raw7_1f)
         ind_calc = 1e9*np.imag(1/dut.y[0,0,0])/2/np.pi/dut.f
         self.assertTrue(np.isclose(ind_calc, 1, rtol=self.rtol))
 
     def test_shortopen(self):
         """
-        First do short de-embedding to remove 2ohm series resistors on each side, then remove
-        open shunt capacitors which are 25fF to ground on each pad, and 10fF pad-to-pad.
-        The resulting network should be a pure inductor of 1nH.
+        After de-embedding, the network is a pure inductor of 1nH.
+        Test that this is true at a spot frequency.
         """
         dut = self.dm_so.deembed(self.raw2_1f)
         ind_calc = 1e9*np.imag(1/dut.y[0,0,0])/2/np.pi/dut.f
         self.assertTrue(np.isclose(ind_calc, 1, rtol=self.rtol))
 
+    def test_short(self):
+        """
+        After de-embedding, the network is a pure inductor of 1nH.
+        Test that this is true at a spot frequency.
+        """
+        dut = self.dm_s.deembed(self.raw8_1f)
+        ind_calc = 1e9*np.imag(1/dut.y[0,0,0])/2/np.pi/dut.f
+        self.assertTrue(np.isclose(ind_calc, 1, rtol=self.rtol))
+
     def test_splitpi(self):
         """
         After de-embedding, the network is a pure inductor of 1nH.
@@ -285,12 +324,58 @@ class DeembeddingTestCase(unittest.TestCase):
         thru.
         Test that this thru has S21 amplitude and phase smaller than a limit. 
         """
-        self.s2xthru = rf.Network(os.path.join(self.test_dir, 's2xthru.s2p'))
-        self.fdf = rf.Network(os.path.join(self.test_dir, 'fdf.s2p'))
-        self.dm_nzc = rf.IEEEP370_SE_NZC_2xThru(dummy_2xthru = self.s2xthru, 
+        s2xthru = rf.Network(os.path.join(self.test_dir, 's2xthru.s2p'))
+        dm_nzc = rf.IEEEP370_SE_NZC_2xThru(dummy_2xthru = s2xthru, 
                                         name = '2xthru')
-        residuals = self.dm_nzc.s_side1.inv ** \
-            self.s2xthru ** self.dm_nzc.s_side2.flipped().inv
+        residuals = dm_nzc.deembed(s2xthru)
+        # insertion loss magnitude deviate from 1.0 from less than 0.1 dB
+        il_mag = 20.*np.log10(np.abs(residuals.s[:, 1, 0] + 1e-12))
+        self.assertTrue(np.max(np.abs(il_mag)) <= 0.1, 'residual IL magnitude')
+        # insertion loss phase deviate from 0 degree from less than 1 degree
+        il_phase = np.angle(residuals.s[:, 1, 0]) * 180/np.pi
+        self.assertTrue(np.max(np.abs(il_phase)) <= 1.0, 'residual IL Phase')
+        
+    def test_IEEEP370_SE_NZC_2xThru_with_dc(self):
+        """
+        Test test_IEEEP370_SE_NZC_2xThru_with_dc.
+
+        After de-embedding fixtures model from 2xtru, the network is a perfect
+        thru.
+        Test that this thru has S21 amplitude and phase smaller than a limit. 
+        """
+        s2xthru = rf.Network(os.path.join(self.test_dir, 's2xthru.s2p'))
+        # interpolate to dc
+        s2xthru_dc = s2xthru.extrapolate_to_dc(kind='linear')
+        with pytest.warns(RuntimeWarning, match="DC point detected"):
+            dm_nzc = rf.IEEEP370_SE_NZC_2xThru(dummy_2xthru = s2xthru_dc, 
+                                        name = '2xthru')
+        residuals = dm_nzc.deembed(s2xthru_dc)
+        # insertion loss magnitude deviate from 1.0 from less than 0.1 dB
+        il_mag = 20.*np.log10(np.abs(residuals.s[:, 1, 0] + 1e-12))
+        self.assertTrue(np.max(np.abs(il_mag)) <= 0.1, 'residual IL magnitude')
+        # insertion loss phase deviate from 0 degree from less than 1 degree
+        il_phase = np.angle(residuals.s[:, 1, 0]) * 180/np.pi
+        self.assertTrue(np.max(np.abs(il_phase)) <= 1.0, 'residual IL Phase')
+        
+    def test_IEEEP370_SE_NZC_2xThru_with_interpolation(self):
+        """
+        Test test_IEEEP370_SE_NZC_2xThru_with_interpolation.
+        While interpolation should be avoided, it keep usefull in some cases,
+        e.g. when the measurement frequency axis was not well formated but
+        a result is still needed and some deviation acceptable.
+        After de-embedding fixtures model from 2xtru, the network is a perfect
+        thru.
+        Test that this thru has S21 amplitude and phase smaller than a limit.
+        """
+        s2xthru = rf.Network(os.path.join(self.test_dir, 's2xthru.s2p'))
+        # with non-hormonic sweep
+        nonuniform_freq = rf.Frequency(s2xthru.f[0], s2xthru.f[-1], 
+                                       npoints=len(s2xthru)-10, unit='Hz')
+        s2xthru_nu = s2xthru.interpolate(nonuniform_freq)
+        with pytest.warns(RuntimeWarning, match="Non-uniform frequency vector detected"):
+            dm_nzc_nu = rf.IEEEP370_SE_NZC_2xThru(dummy_2xthru = s2xthru_nu, 
+                                                name = '2xthru')
+        residuals = dm_nzc_nu.deembed(s2xthru_nu)
         # insertion loss magnitude deviate from 1.0 from less than 0.1 dB
         il_mag = 20.*np.log10(np.abs(residuals.s[:, 1, 0] + 1e-12))
         self.assertTrue(np.max(np.abs(il_mag)) <= 0.1, 'residual IL magnitude')
@@ -306,17 +391,79 @@ class DeembeddingTestCase(unittest.TestCase):
         thru.
         Test that this thru has S21 amplitude and phase smaller than a limit. 
         """
-        self.s2xthru = rf.Network(os.path.join(self.test_dir, 's2xthru.s2p'))
-        self.fdf = rf.Network(os.path.join(self.test_dir, 'fdf.s2p'))
-        self.dm_zc  = rf.IEEEP370_SE_ZC_2xThru(dummy_2xthru = self.s2xthru, 
-                                       dummy_fix_dut_fix = self.fdf, 
+        s2xthru = rf.Network(os.path.join(self.test_dir, 's2xthru.s2p'))
+        fdf = rf.Network(os.path.join(self.test_dir, 'fdf.s2p'))
+        dm_zc  = rf.IEEEP370_SE_ZC_2xThru(dummy_2xthru = s2xthru, 
+                                       dummy_fix_dut_fix = fdf, 
                                        bandwidth_limit = 10e9, 
                                        pullback1 = 0, pullback2 = 0,
                                        leadin = 0,
                                        NRP_enable = False,
                                        name = 'zc2xthru')
-        residuals = self.dm_zc.s_side1.inv ** \
-            self.s2xthru ** self.dm_zc.s_side2.flipped().inv
+        residuals = dm_zc.deembed(s2xthru)
+        # insertion loss magnitude deviate from 1.0 from less than 0.2 dB
+        il_mag = 20.*np.log10(np.abs(residuals.s[:, 1, 0] + 1e-12))
+        self.assertTrue(np.max(np.abs(il_mag)) <= 0.2, 'residual IL magnitude')
+        # insertion loss phase deviate from 0 degree from less than 45 degree
+        # too much tolerance here allowed as for now
+        il_phase = np.angle(residuals.s[:, 1, 0]) * 180/np.pi
+        self.assertTrue(np.max(np.abs(il_phase)) <= 2.0, 'residual IL Phase')
+        
+    def test_IEEEP370_SE_ZC_2xThru_with_dc(self):
+        """
+        Test test_IEEEP370_SE_ZC_2xThru_with_dc.
+
+        After de-embedding fixtures model from 2xtru, the network is a perfect
+        thru.
+        Test that this thru has S21 amplitude and phase smaller than a limit. 
+        """
+        s2xthru = rf.Network(os.path.join(self.test_dir, 's2xthru.s2p'))
+        fdf = rf.Network(os.path.join(self.test_dir, 'fdf.s2p'))
+        s2xthru_dc = s2xthru.extrapolate_to_dc(kind='linear')
+        fdf_dc = fdf.extrapolate_to_dc(kind='linear')
+        with pytest.warns(RuntimeWarning, match="DC point detected"):
+            dm_zc  = rf.IEEEP370_SE_ZC_2xThru(dummy_2xthru = s2xthru_dc, 
+                                        dummy_fix_dut_fix = fdf_dc, 
+                                        bandwidth_limit = 10e9, 
+                                        pullback1 = 0, pullback2 = 0,
+                                        leadin = 0,
+                                        NRP_enable = False,
+                                        name = 'zc2xthru')
+        residuals = dm_zc.deembed(s2xthru_dc)
+        # insertion loss magnitude deviate from 1.0 from less than 0.2 dB
+        il_mag = 20.*np.log10(np.abs(residuals.s[:, 1, 0] + 1e-12))
+        self.assertTrue(np.max(np.abs(il_mag)) <= 0.2, 'residual IL magnitude')
+        # insertion loss phase deviate from 0 degree from less than 45 degree
+        # too much tolerance here allowed as for now
+        il_phase = np.angle(residuals.s[:, 1, 0]) * 180/np.pi
+        self.assertTrue(np.max(np.abs(il_phase)) <= 2.0, 'residual IL Phase')
+        
+    def test_IEEEP370_SE_ZC_2xThru_with_interpolation(self):
+        """
+        Test test_IEEEP370_SE_ZC_2xThru_with_interpolation.
+        While interpolation should be avoided, it keep usefull in some cases,
+        e.g. when the measurement frequency axis was not well formated but
+        a result is still needed and some deviation acceptable.
+        After de-embedding fixtures model from 2xtru, the network is a perfect
+        thru.
+        Test that this thru has S21 amplitude and phase smaller than a limit.
+        """
+        s2xthru = rf.Network(os.path.join(self.test_dir, 's2xthru.s2p'))
+        fdf = rf.Network(os.path.join(self.test_dir, 'fdf.s2p'))
+        # with non-hormonic sweep
+        nonuniform_freq = rf.Frequency(s2xthru.f[0], s2xthru.f[-1], 
+                                       npoints=len(s2xthru)-10, unit='Hz')
+        s2xthru_nu = s2xthru.interpolate(nonuniform_freq)
+        fdf_nu = fdf.interpolate(nonuniform_freq)
+        with pytest.warns(RuntimeWarning, match="Non-uniform frequency vector detected"):
+            dm_zc_nu  = rf.IEEEP370_SE_ZC_2xThru(dummy_2xthru = s2xthru_nu, 
+                                        dummy_fix_dut_fix = fdf_nu, 
+                                        bandwidth_limit = 10e9, 
+                                        pullback1 = 0, pullback2 = 0,
+                                        leadin = 0,
+                                        NRP_enable = False,
+                                        name = 'zc2xthru')
+        residuals = dm_zc_nu.deembed(s2xthru_nu)
         # insertion loss magnitude deviate from 1.0 from less than 0.2 dB
         il_mag = 20.*np.log10(np.abs(residuals.s[:, 1, 0] + 1e-12))
         self.assertTrue(np.max(np.abs(il_mag)) <= 0.2, 'residual IL magnitude')
@@ -327,7 +474,7 @@ class DeembeddingTestCase(unittest.TestCase):
         
     def test_IEEEP370_SE_NZC_2xThru_no_180deg_phase_jumps(self):
         """
-        Test test_IEEEP370_SE_NZC_2xThru.
+        Test test_IEEEP370_SE_NZC_2xThru_no_180deg_phase_jumps.
 
         After de-embedding fixtures model from 2xtru, the network is a perfect
         thru.
@@ -335,47 +482,21 @@ class DeembeddingTestCase(unittest.TestCase):
         e10 could lead to 180° jumps into the side models s21 phase.
         Check this not happens.
         """
-        self.s2xthru = rf.Network(os.path.join(self.test_dir, 's2xthru.s2p'))
-        self.fdf = rf.Network(os.path.join(self.test_dir, 'fdf.s2p'))
+        s2xthru = rf.Network(os.path.join(self.test_dir, 's2xthru.s2p'))
         # with phase noise
         # add enough phase noise to trigger phase jumps with original
         # implementation, but small enough to keep within 1° limit line
-        self.s2xthru_pn = self.s2xthru.copy()
-        self.s2xthru_pn.add_noise_polar(0.0002, 0.2)
-        self.dm_nzc_pn = rf.IEEEP370_SE_NZC_2xThru(dummy_2xthru = self.s2xthru_pn, 
+        s2xthru_pn = s2xthru.copy()
+        s2xthru_pn.add_noise_polar(0.0002, 0.2)
+        dm_nzc_pn = rf.IEEEP370_SE_NZC_2xThru(dummy_2xthru = s2xthru_pn, 
                                         name = '2xthru')
-        residuals = self.dm_nzc_pn.s_side1.inv ** \
-            self.s2xthru_pn ** self.dm_nzc_pn.s_side2.flipped().inv
+        residuals = dm_nzc_pn.deembed(s2xthru_pn)
         # insertion loss magnitude deviate from 1.0 from less than 0.1 dB
         il_mag = 20.*np.log10(np.abs(residuals.s[:, 1, 0] + 1e-12))
         self.assertTrue(np.max(np.abs(il_mag)) <= 0.1, 'residual IL magnitude')
         # insertion loss phase deviate from 0 degree from less than 1 degree
         il_phase = np.angle(residuals.s[:, 1, 0]) * 180/np.pi
         self.assertTrue(np.max(np.abs(il_phase)) <= 1.0, 'residual IL Phase')
-        
-    def test_IEEEP370_interpolation_not_implemented(self):
-        """
-        Test if IEEEP370 methods fails when interpolation is required.
-        TODO: add interpolation handling
-        """
-        self.s2xthru = rf.Network(os.path.join(self.test_dir, 's2xthru.s2p'))
-        self.fdf = rf.Network(os.path.join(self.test_dir, 'fdf.s2p'))
-        # with non-uniform frequencies
-        nonuniform_freq = rf.Frequency(self.s2xthru.f[0], self.s2xthru.f[-1], 
-                                       npoints=len(self.s2xthru), unit='Hz', 
-                                       sweep_type='log')
-        self.s2xthru_nu = self.s2xthru.interpolate(nonuniform_freq)
-        self.fdf_nu = self.fdf.interpolate(nonuniform_freq)
-        with self.assertRaises(NotImplementedError) as context:
-            self.dm_nzc_nu = rf.IEEEP370_SE_NZC_2xThru(dummy_2xthru = self.s2xthru_nu, 
-                                            name = '2xthru')
-            self.dm_zc_nu  = rf.IEEEP370_SE_ZC_2xThru(dummy_2xthru = self.s2xthru_nu, 
-                                           dummy_fix_dut_fix = self.fdf_nu, 
-                                           bandwidth_limit = 10e9, 
-                                           pullback1 = 0, pullback2 = 0,
-                                           leadin = 0,
-                                           NRP_enable = False,
-                                           name = 'zc2xthru')
 
 if __name__ == "__main__":
     # Launch all tests
