@@ -1,4 +1,3 @@
-
 import unittest
 import os
 import numpy as npy
@@ -35,7 +34,7 @@ class TouchstoneTestCase(unittest.TestCase):
 
         self.assertTrue((f == f_true).all())
         self.assertTrue((s == s_true).all())
-        self.assertTrue((z0 == z0_true))
+        self.assertTrue(z0 == z0_true)
 
     def test_read_with_special_encoding(self):
         """
@@ -70,7 +69,7 @@ class TouchstoneTestCase(unittest.TestCase):
 
         self.assertTrue((f == f_true).all())
         self.assertTrue((s == s_true).all())
-        self.assertTrue((z0 == z0_true))
+        self.assertTrue(z0 == z0_true)
 
     def test_get_sparameter_data(self):
         """
@@ -140,6 +139,12 @@ class TouchstoneTestCase(unittest.TestCase):
                     msg='Field %s does not match. Expected "%s", got "%s"'%(
                         k, str(expected_sp_db[k]), str(sp_db[k]))  )
 
+        for k, v in zip(touch.get_sparameter_names(), touch.sparameters.T):
+            if k[0] != 'S':
+                # frequency doesn't match because of Hz vs GHz.
+                continue
+            self.assertTrue(npy.all(expected_sp_ri[k] == v))
+
 
     def test_HFSS_touchstone_files(self):
         """
