@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 coaxial (:mod:`skrf.media.coaxial`)
 ============================================================
@@ -14,7 +13,7 @@ A coaxial transmission line defined from its electrical or geometrical/physical
 
 #from copy import deepcopy
 from scipy.constants import  epsilon_0, mu_0, pi, c
-from numpy import sqrt, log, real, imag, exp, expm1, size
+from numpy import sqrt, log, real, imag, exp, expm1, size, array
 from ..tlineFunctions import surface_resistivity, skin_depth
 from .distributedCircuit import DistributedCircuit
 from .media import Media, DefinedGammaZ0
@@ -122,7 +121,7 @@ class Coaxial(DistributedCircuit, Media):
 
         """
         # test size of parameters
-        if size(att) not in (1, size(frequency)):
+        if size(array(att, dtype="object")) not in (1, size(array(frequency, dtype="object"))):
             raise ValueError('Attenuation should be scalar or of same size that the frequency.')
     
         # create gamma
