@@ -1,4 +1,3 @@
-
 """
 media (:mod:`skrf.media.media`)
 ========================================
@@ -13,6 +12,7 @@ Media class.
 
 """
 from numbers import Number
+from pathlib import Path
 import warnings
 
 import numpy as npy
@@ -407,7 +407,7 @@ class Media(ABC):
         if z0 is None:
             z0 = self.z0
         elif isinstance(z0, str):
-            z0 = parse_z0(z0)*self.z0
+            z0 = npy.ones(result.s.shape[:2]) * parse_z0(z0)
 
         if z0_norm:
             z0 = z0*self.z0
@@ -594,7 +594,7 @@ class Media(ABC):
         Parameters
         ----------
         L : number, array
-            Inductance, in Henrys. If this is an array, must be of
+            Inductance, in Henries. If this is an array, must be of
             same length as frequency vector.
         \*\*kwargs : key word arguments
             passed to :func:`match`, which is called initially to create a
@@ -836,8 +836,10 @@ class Media(ABC):
 
         kwargs.update({'z0':z0})
         s_def = kwargs.pop('s_def', S_DEF_DEFAULT)
-        # Need to use either traveling or pseudo definition here
-        # for the network to match traveling waves.
+        # The use of either traveling or pseudo waves s-parameters definition
+        # is required here.
+        # The definition of the reflection coefficient for power waves has
+        # conjugation. 
         result = self.match(nports=2, s_def='traveling', **kwargs)
 
         theta = self.electrical_length(self.to_meters(d=d, unit=unit))
@@ -847,17 +849,16 @@ class Media(ABC):
         result.s = \
                 npy.array([[s11, s21],[s21,s11]]).transpose().reshape(-1,2,2)
 
-        if embed:
+        if embed and self.z0 is not None:
             # warns of future deprecation
             warnings.warn('In a future version,`embed` will be deprecated.\n'
                           'The line and media port impedance z0 and '
                           'characteristic impedance Z0 will be used instead '
                           'to determine if the line has to be renormalized.',
               FutureWarning, stacklevel = 2)
-            # Use the same s_def here as the line to avoid changing it during
-            # cascade.
-            result = self.thru(s_def='traveling')**result**self.thru(s_def='traveling')
-        result.renormalize(result.z0, s_def=s_def)
+            result.renormalize(self.z0, s_def=s_def)
+        else:
+            result.renormalize(result.z0, s_def=s_def)
 
         return result
 
@@ -902,7 +903,8 @@ class Media(ABC):
         delay_short
         delay_open
         """
-        return self.line(d=d, unit=unit, **kwargs) ** self.load(Gamma0=Gamma0, **kwargs)
+        return self.line(d=d, unit=unit, **kwargs) ** self.load(Gamma0=Gamma0,
+                                                                **kwargs)
 
     def delay_short(self, d: Number, unit: str = 'deg', **kwargs) -> Network:
         r"""
@@ -1022,7 +1024,7 @@ class Media(ABC):
         shunt_capacitor
         shunt_inductor
         """
-        return self.shunt(self.delay_load(*args, **kwargs))
+        return self.shunt(self.delay_load(*args, **kwargs), **kwargs)
 
     def shunt_delay_open(self,*args,**kwargs) -> Network:
         r"""
@@ -1052,7 +1054,7 @@ class Media(ABC):
         shunt_capacitor
         shunt_inductor
         """
-        return self.shunt(self.delay_open(*args, **kwargs))
+        return self.shunt(self.delay_open(*args, **kwargs), **kwargs)
 
     def shunt_delay_short(self, *args, **kwargs) -> Network:
         r"""
@@ -1082,7 +1084,7 @@ class Media(ABC):
         shunt_capacitor
         shunt_inductor
         """
-        return self.shunt(self.delay_short(*args, **kwargs))
+        return self.shunt(self.delay_short(*args, **kwargs), **kwargs)
 
     def shunt_capacitor(self, C: NumberLike, **kwargs) -> Network:
         r"""
@@ -1114,7 +1116,8 @@ class Media(ABC):
         shunt_delay_short
         shunt_inductor
         """
-        return self.shunt(self.capacitor(C=C, **kwargs) ** self.short())
+        return self.shunt(self.capacitor(C=C, **kwargs) ** 
+                          self.short(**kwargs), **kwargs)
 
     def shunt_inductor(self, L: NumberLike, **kwargs) -> Network:
         r"""
@@ -1123,7 +1126,7 @@ class Media(ABC):
         Parameters
         ----------
         L : number, array-like
-            Inductance in Farads.
+            Inductance in Henries.
         \*\*kwargs : arguments, keyword arguments
             passed to func:`inductor`
 
@@ -1146,7 +1149,8 @@ class Media(ABC):
         shunt_delay_short
         shunt_capacitor
         """
-        return self.shunt(self.inductor(L=L, **kwargs) ** self.short())
+        return self.shunt(self.inductor(L=L, **kwargs) **
+                          self.short(**kwargs), **kwargs)
 
     def attenuator(self, s21: NumberLike, db: bool = True, d: Number = 0,
                    unit: str = 'deg', name: str = '', **kwargs) -> Network:
@@ -1436,7 +1440,7 @@ class DefinedGammaZ0(Media):
     def __init__(self, frequency: Union[Frequency, None] = None,
                  z0: Union[NumberLike, None] = None, Z0: NumberLike = 50,
                  gamma: NumberLike = 1j):
-        super(DefinedGammaZ0, self).__init__(frequency=frequency,
+        super().__init__(frequency=frequency,
                                              z0=z0)
         self.gamma= gamma
         self.Z0 = Z0
@@ -1459,17 +1463,20 @@ class DefinedGammaZ0(Media):
         write_csv
         """
         try:
-            f = open(filename)
+            fid = open(filename)
         except(TypeError):
             # they may have passed a file
-            f = filename
+            fid = filename
 
-        header = f.readline()
+        header = fid.readline()
         # this is not the correct way to do this ... but whatever
         f_unit = header.split(',')[0].split('[')[1].split(']')[0]
 
         f,z_re,z_im,g_re,g_im,pz_re,pz_im = \
-                npy.loadtxt(f,  delimiter=',').T
+                npy.loadtxt(fid,  delimiter=',').T
+
+        if isinstance(filename, (str, Path)):
+            fid.close()
 
         return cls(
             frequency = Frequency.from_f(f, unit=f_unit),
