@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 .. module:: skrf.networkSet
 
@@ -45,21 +44,24 @@ NetworkSet Utilities
 
 """
 import zipfile
-import numpy as npy
-from numbers import Number
-from typing import Union, Any, Mapping, TextIO
 from io import BytesIO
+from numbers import Number
+from typing import Any, Mapping, TextIO, Union
+
+import numpy as npy
 from scipy.interpolate import interp1d
-from . network import Network, Frequency, PRIMARY_PROPERTIES, COMPONENT_FUNC_DICT
+
 from . import mathFunctions as mf
-from . util import now_string_2_dt
+from .network import (COMPONENT_FUNC_DICT, PRIMARY_PROPERTIES, Frequency,
+                      Network)
+from .util import now_string_2_dt
 
 try:
     from numpy.typing import ArrayLike
 except ImportError:
     ArrayLike = Any
 
-class NetworkSet(object):
+class NetworkSet:
     """
     A set of Networks.
 
@@ -207,7 +209,7 @@ class NetworkSet(object):
             self.__add_a_element_wise_method(network_method_name)
 
         for operator_name in \
-                ['__pow__','__floordiv__','__mul__','__div__','__add__','__sub__']:
+                ['__pow__','__floordiv__','__mul__','__truediv__','__add__','__sub__']:
             self.__add_a_operator(operator_name)
 
     @classmethod
@@ -280,7 +282,7 @@ class NetworkSet(object):
         >>> my_set = rf.NetworkSet.from_dir('./data/')
 
         """
-        from . io.general import read_all_networks
+        from .io.general import read_all_networks
         return cls(read_all_networks(dir), *args, **kwargs)
 
     @classmethod
@@ -333,7 +335,7 @@ class NetworkSet(object):
         write_mdif : Convert a NetworkSet to a Generalized MDIF file.
 
         """
-        from . io import Mdif
+        from .io import Mdif
         return Mdif(file).to_networkset()
 
     @classmethod
@@ -355,7 +357,7 @@ class NetworkSet(object):
         Citi
 
         """
-        from . io import Citi
+        from .io import Citi
         return Citi(file).to_networkset()
 
     def __add_a_operator(self, operator_name):
@@ -533,7 +535,7 @@ class NetworkSet(object):
             and the Networks as values.
 
         """
-        return dict([(k.name, k) for k in self.ntwk_set])
+        return {k.name: k for k in self.ntwk_set}
 
     def to_s_dict(self):
         """
@@ -787,8 +789,8 @@ class NetworkSet(object):
 
 
         """
-        from scipy import stats
         from numpy import frompyfunc
+        from scipy import stats
 
         gimme_norm = lambda x: stats.norm(loc=0,scale=x).rvs(1)[0]
         ugimme_norm = frompyfunc(gimme_norm,1,1)
@@ -908,7 +910,7 @@ class NetworkSet(object):
 
         """
         # this import is delayed until here because of a circular dependency
-        from . io.general import write
+        from .io.general import write
 
         if file is None:
             if self.name is None:
@@ -932,7 +934,7 @@ class NetworkSet(object):
         skrf.io.general.network_2_spreadsheet
 
         """
-        from . io.general import networkset_2_spreadsheet
+        from .io.general import networkset_2_spreadsheet
         networkset_2_spreadsheet(self, *args, **kwargs)
 
     def write_mdif(self,
@@ -965,7 +967,7 @@ class NetworkSet(object):
         params_types : parameters types
 
         """
-        from . io import Mdif
+        from .io import Mdif
         Mdif.write(ns=self, filename=filename, values=values, 
                              data_types=data_types, comments=comments)
 
@@ -979,15 +981,15 @@ class NetworkSet(object):
         >>> df.to_excel('output.xls')  # see Pandas docs for more info
 
         """
-        from pandas import DataFrame, Series, Index
+        from pandas import DataFrame, Index, Series
         index = Index(
             self[0].frequency.f_scaled,
             name='Freq(%s)'%self[0].frequency.unit
             )
         df = DataFrame(
-            dict([('%s'%(k.name),
-                Series(k.__getattribute__(attr)[:,m,n],index=index))
-                for k in self]),
+            {'%s'%(k.name):
+                Series(k.__getattribute__(attr)[:,m,n],index=index)
+                for k in self},
             index = index,
             )
         return df
@@ -1410,4 +1412,3 @@ def tuner_constellation(name='tuner', singlefreq=76, Z0=50, r_lin = 9, phi_lin=2
         return TNW, x,y,g
     else :
         return x,y,g
-
