@@ -1,13 +1,15 @@
 import unittest
+import pytest
 import skrf
 import numpy as np
 import tempfile
 import os
-import warnings
+import sys
 
 
 class VectorFittingTestCase(unittest.TestCase):
 
+    @pytest.mark.filterwarnings('ignore::UserWarning')
     def test_ringslot_with_proportional(self):
         # perform the fit
         nw = skrf.data.ring_slot
@@ -15,6 +17,7 @@ class VectorFittingTestCase(unittest.TestCase):
         vf.vector_fit(n_poles_real=2, n_poles_cmplx=0, fit_proportional=True, fit_constant=True)
         self.assertLess(vf.get_rms_error(), 0.02)
 
+    @pytest.mark.filterwarnings('ignore::UserWarning')
     def test_ringslot_default_log(self):
         # perform the fit without proportional term
         nw = skrf.data.ring_slot
@@ -22,6 +25,7 @@ class VectorFittingTestCase(unittest.TestCase):
         vf.vector_fit(n_poles_real=4, n_poles_cmplx=0, init_pole_spacing='log')
         self.assertLess(vf.get_rms_error(), 0.01)
 
+    @pytest.mark.filterwarnings('ignore::UserWarning')
     def test_ringslot_without_prop_const(self):
         # perform the fit without proportional term
         nw = skrf.data.ring_slot
@@ -29,6 +33,15 @@ class VectorFittingTestCase(unittest.TestCase):
         vf.vector_fit(n_poles_real=4, n_poles_cmplx=0, fit_proportional=False, fit_constant=False)
         self.assertLess(vf.get_rms_error(), 0.01)
 
+    @pytest.mark.filterwarnings('ignore::UserWarning')
+    def test_ringslot_custompoles(self):
+        # perform the fit with custom initial poles
+        nw = skrf.data.ring_slot
+        vf = skrf.vectorFitting.VectorFitting(nw)
+        vf.poles = 2 * np.pi * np.array([-100e9, -10e9 + 100e9j])
+        vf.vector_fit(init_pole_spacing='custom')
+        self.assertLess(vf.get_rms_error(), 0.01)
+
     def test_190ghz_measured(self):
         # perform the fit without proportional term
         nw = skrf.network.Network('./doc/source/examples/vectorfitting/190ghz_tx_measured.S2P')
@@ -38,12 +51,25 @@ class VectorFittingTestCase(unittest.TestCase):
 
     def test_no_convergence(self):
         # perform a bad fit that does not converge and check if a RuntimeWarning is given
-        with warnings.catch_warnings(record=True) as warning:
-            nw = skrf.network.Network('./doc/source/examples/vectorfitting/190ghz_tx_measured.S2P')
-            vf = skrf.vectorFitting.VectorFitting(nw)
+        nw = skrf.network.Network('./doc/source/examples/vectorfitting/190ghz_tx_measured.S2P')
+        vf = skrf.vectorFitting.VectorFitting(nw)
+        
+        with pytest.warns(RuntimeWarning) as record:
             vf.vector_fit(n_poles_real=0, n_poles_cmplx=5, fit_proportional=False, fit_constant=True)
-            self.assertEqual(warning[-1].category, RuntimeWarning)
+        
+        assert len(record) == 1
 
+    def test_dc(self):
+        # perform the fit on data including a dc sample (0 Hz)
+        nw = skrf.Network('./skrf/tests/cst_example_4ports.s4p')
+        vf = skrf.VectorFitting(nw)
+        vf.vector_fit(n_poles_real=3, n_poles_cmplx=0)
+        # quality of the fit is not important in this test; it only needs to finish
+        self.assertLess(vf.get_rms_error(), 0.2)
+
+    @pytest.mark.skipif(
+        "matplotlib" not in sys.modules, 
+        reason="Spice subcircuit uses Engformatter which is not available without matplotlib.")
     def test_spice_subcircuit(self):
         # fit ring slot example network
         nw = skrf.data.ring_slot
@@ -51,27 +77,34 @@ class VectorFittingTestCase(unittest.TestCase):
         vf.vector_fit(n_poles_real=4, n_poles_cmplx=0, fit_constant=True, fit_proportional=True)
 
         # write equivalent SPICE subcircuit to tmp file
-        tmp_file = tempfile.NamedTemporaryFile(suffix='.sp')
-        tmp_file.close()  # tmp_file.name can be used to open the file a second time on Linux but not on Windows
-        vf.write_spice_subcircuit_s(tmp_file.name)
+        tmp_file = tempfile.NamedTemporaryFile(suffix='.sp', delete=False)
+        name = tmp_file.name
+        tmp_file.close()
+        vf.write_spice_subcircuit_s(name)
 
         # written tmp file should contain 69 lines
-        n_lines = len(open(tmp_file.name, 'r').readlines())
+        with open(name) as f:
+            n_lines = len(f.readlines())
         self.assertEqual(n_lines, 69)
+        os.remove(name)
 
     def test_read_write_npz(self):
         # fit ring slot example network
         nw = skrf.data.ring_slot
         vf = skrf.vectorFitting.VectorFitting(nw)
-        vf.vector_fit(n_poles_real=3, n_poles_cmplx=0)
+
+        with pytest.warns(UserWarning) as record:
+            vf.vector_fit(n_poles_real=3, n_poles_cmplx=0)
+
+        assert len(record) == 1
 
         # export (write) fitted parameters to .npz file in tmp directory
-        tmp_dir = tempfile.TemporaryDirectory()
-        vf.write_npz(tmp_dir.name)
+        with  tempfile.TemporaryDirectory() as name:
+            vf.write_npz(name)
 
-        # create a new vector fitting instance and import (read) those fitted parameters
-        vf2 = skrf.vectorFitting.VectorFitting(nw)
-        vf2.read_npz(os.path.join(tmp_dir.name, 'coefficients_{}.npz'.format(nw.name)))
+            # create a new vector fitting instance and import (read) those fitted parameters
+            vf2 = skrf.vectorFitting.VectorFitting(nw)
+            vf2.read_npz(os.path.join(name, f'coefficients_{nw.name}.npz'))
 
         # compare both sets of parameters
         self.assertTrue(np.allclose(vf.poles, vf2.poles))
@@ -79,9 +112,9 @@ class VectorFittingTestCase(unittest.TestCase):
         self.assertTrue(np.allclose(vf.proportional_coeff, vf2.proportional_coeff))
         self.assertTrue(np.allclose(vf.constant_coeff, vf2.constant_coeff))
 
+    @pytest.mark.skipif("matplotlib" in sys.modules, reason="Raise Error only if matplotlib is not installed.")
     def test_matplotlib_missing(self):
         vf = skrf.vectorFitting.VectorFitting(skrf.data.ring_slot)
-        skrf.vectorFitting.mplt = None
         with self.assertRaises(RuntimeError):
             vf.plot_convergence()
 
